﻿using System;

namespace RecrownedGTK.Tools.CommandProcessor.Commands
{
    public class HelpCommand : EngineCommand
    {
        CommandEngine commandEngine;

        public HelpCommand(CommandEngine commandEngine) : base("help [command] [arg]", "help")
        {
            this.commandEngine = commandEngine;
        }


        public override bool Run(IUserInput userInput, IUserOutput userOutput, string[] arguments)
        {
            if (arguments != null)
            {
                if (commandEngine.ContainsCommand(arguments[0]))
                {
                    if (arguments.Length < 2) userOutput.WriteLine(commandEngine.GetCommand(arguments[0]).Help(null));
                    for (int i = 1; i < arguments.Length; i++)
                    {
                        userOutput.WriteLine(commandEngine.GetCommand(arguments[0]).Help(arguments[i]));
                    }
                }
                else
                {
                    return false;
                }
            }
            else
            {
                userOutput.WriteLine("Tools for RecrownedGTK library. Possible commands are as follows:\n");
                foreach (EngineCommand engineCommand in commandEngine.commands)
                {
                    for (int i = 0; i < engineCommand.InvokeStrings.Length; i++)
                    {
                        userOutput.Write(engineCommand.InvokeStrings[i]);
                        if (i + 1 < engineCommand.InvokeStrings.Length)
                        {
                            userOutput.Write(", ");
                        }
                    }
                    userOutput.Write(": ");
                    userOutput.Write(engineCommand.Help());
                    userOutput.Write("\n");
                    userOutput.WriteLine("--------");
                }
            }
            return true;
        }
    }
}
