﻿using Newtonsoft.Json;
using RecrownedGTK.AssetsSystem.Information;
using RecrownedGTK.Tools.CommandProcessor;
using System;
using System.IO;

namespace RecrownedGTK.Tools.NinePatchTools
{
    public class NinePatchCommand : EngineCommand
    {

        public NinePatchCommand() : base("Generates a 9 patch file for a given image.", "9p", "ninepatch", "9patch")
        {
            arguments = new EngineCommandArgument[6];

            arguments[0] = new EngineCommandArgument("-i", "defines the path to the texture to be used for the nine patch.", true);
            arguments[1] = new EngineCommandArgument("-o", "defines path of output file.");
            arguments[2] = new EngineCommandArgument("-l", "left patch.", true);
            arguments[3] = new EngineCommandArgument("-r", "right patch.", true);
            arguments[4] = new EngineCommandArgument("-t", "top patch.", true);
            arguments[5] = new EngineCommandArgument("-b", "bottom patch.", true);
        }

        public override bool Run(IUserInput userInput = null, IUserOutput userOutput = null, string[] arguments = null)
        {
            if (arguments == null) throw new ArgumentException("Missing arguments. Type \"help 9p\" for more information.");
            int leftBound = 0, rightBound = 0, topBound = 0, bottomBound = 0;
            string imagePath, outPath;
            if (IndexOfArgument("-i", arguments) + 1 >= arguments.Length) userOutput.WriteLine("Missing -i path after argument.");
            imagePath = arguments[IndexOfArgument("-i", arguments) + 1];
            if (!File.Exists(imagePath)) userOutput.WriteLine("Input file does not exist at " + imagePath + ".");

            if (HasArgument(base.arguments[1], arguments))
            {
                if (IndexOfArgument("-o", arguments) + 1 >= arguments.Length) userOutput.WriteLine("Missing -o path after argument.");
                outPath = arguments[IndexOfArgument("-o", arguments) + 1];
            } else
            {
                outPath = imagePath.Substring(0, imagePath.Length - Path.GetExtension(imagePath).Length);
            }
            if (IndexOfArgument("-l", arguments) + 1 >= arguments.Length || !int.TryParse(arguments[IndexOfArgument("-l", arguments) + 1], out leftBound)) throw new ArgumentException("Missing -l argument bound.");
            if (IndexOfArgument("-r", arguments) + 1 >= arguments.Length || !int.TryParse(arguments[IndexOfArgument("-r", arguments) + 1], out rightBound)) throw new ArgumentException("Missing -r argument bound.");
            if (IndexOfArgument("-t", arguments) + 1 >= arguments.Length || !int.TryParse(arguments[IndexOfArgument("-t", arguments) + 1], out topBound)) throw new ArgumentException("Missing -u argument bound.");
            if (IndexOfArgument("-b", arguments) + 1 >= arguments.Length || !int.TryParse(arguments[IndexOfArgument("-b", arguments) + 1], out bottomBound)) throw new ArgumentException("Missing -d argument bound.");

            NinePatchInfo npData = new NinePatchInfo(Path.GetFileName(imagePath), leftBound, rightBound, bottomBound, topBound);
            string serialized = JsonConvert.SerializeObject(npData, Formatting.Indented);
            string modifiedPath = Directory.GetParent(imagePath) + Path.PathSeparator.ToString() + Path.GetFileNameWithoutExtension(imagePath) + "-texture" + Path.GetExtension(imagePath);
            File.Move(imagePath, modifiedPath);
            File.WriteAllText(outPath + ".9p", serialized);

            userOutput.WriteLine("Done. Written to \"" + outPath + "\" with values: left = " + leftBound + " right = " + rightBound + " top = " + topBound + " bottom = " + bottomBound);
            userOutput.WriteLine("Image renamed to: " + Path.GetFileName(modifiedPath));

            return true;
        }
    }
}
